<?php
// retrieve all the file names within the directories: group1, group2, and group3
$groupNumbers = array(1, 2, 3);
foreach($groupNumbers as $number) {
  $path = "group".$number;
  $files = Centiment::getFiles($path);

  $testcases = array();
  foreach($files as $filename) {
    // upload and ingest the file to the centiment API webservice
    $datasourceXML = Centiment::ingestFile($path, $filename);
    $testcases[] = $datasourceXML;
  }
  
  $outputXML = "<datasources>";
  $outputXML .= implode("\n", $testcases);
  $outputXML .= "</datasources>\n";
  $outFile = "testcases_group".$number.".xml";
  file_put_contents($outFile, $outputXML);
}

// open all the testcase group files as xml
foreach($groupNumbers as $number) {
  $xmlFile = "testcases_group".$number.".xml";
  $datasourcesXML = file_get_contents($xmlFile);
  $sourcesXML = new SimpleXMLElement($datasourcesXML);
  $testcasesXML = $sourcesXML->datasource;
  
  // retrieve the id as the test case that was ingested
  $inferences = array();
  $ids = array();
  foreach($testcasesXML as $testcaseXML) {
    $id = (int)$testcaseXML->id;
    $filename = (string)$testcaseXML->name;
    $ids[] = $id;
    // create the inference and output the results to a file
    $inference = Centiment::createInference($id, $filename);
    $inferences[] = $inference;
  }
  
  // store the inference output file in an output directory specific to each group
    $outputXML = "<inferences>";
    $outputXML .= implode("\n", $inferences);
    $outputXML .= "</inferences>\n";
    $outFile = "inference_group".$number.".xml";
    file_put_contents($outFile, $outputXML);
}


class Centiment {
  
  public static function getFiles($path) {
    $files = scandir($path);
    // only grab the grp{#number}-pt-{#number}.xml.txt (text) files
    $groupFiles = array();
    foreach($files as $filename) {
      $fileparts = pathinfo("./".$path."/".$filename);
      if(isset($fileparts['extension']) && $fileparts['extension'] == "txt") {
        $groupFiles[] = $filename;
      }  
    }
    return $groupFiles;
  }
  
  public static function ingestFile($path, $filename) {
    $parameters = array( "op" => "upload",
       "user_id" => "1",
       "name" => str_replace(".xml.txt", "", $filename),
       "description" => "Test case for suicide mode for ".$filename,
       "type" => "text",
       "share_level" => "1",
       "location" => "@./".$path."/".$filename
    );
    $datasourceXML = Centiment::invoke_service("datasources", $parameters);
    
    return str_replace('<?xml version="1.0" encoding="UTF-8" ?>', "", $datasourceXML);
  }
  
  public static function createInference($id, $filename=null) {
    // invoke the centiment service for inference prediction creation
    $inference_parameters = array( "op" => "vaset",
       "user_id" => "1",
       "model_id" => 10,
       "model2_id" => 11,
       "prediction_description" => "suicidality prediction for ".$filename,
       "share_level" => 1,
       "test_case_id" => $id,
       "test_case_join_key" => "filename_without_extension",
       "prediction_date" => "2013-02-27"
    );

    $inferenceXML = Centiment::invoke_service("inferences", $inference_parameters);
    return str_replace('<?xml version="1.0" encoding="UTF-8" ?>', "", $inferenceXML);
  }

  public static function invoke_service($controller, $parameters = array()) {
    echo "\n executing centiment api $controller service \n";
    $param_str = array();
    echo "building request parameters \n";
    foreach($parameters as $key => $value) {
      echo "$key: $value \n";
      $param_str[] = "$key=".urlencode($value);
    }
    $params = implode("&", $param_str);
  
    $post_url = "http://predictus.net:18000/api/v1/centiment/$controller";
    $url = $post_url."?".$params;
    $ch = curl_init();
    curl_setopt($ch, CURLOPT_URL, $url);
    curl_setopt($ch, CURLOPT_POST, 1);
    curl_setopt($ch, CURLOPT_POSTFIELDS, $parameters);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
    curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 1);

    echo "\n invoking url $url \n";
    $output = curl_exec($ch);
    echo "api response: \n $output";
    curl_close($ch);
    return $output;
  }
}

?>