/*
 * moses3/comboreduct/ant_combo_vocabulary/ant_action_symbol.cc
 *
 * Copyright (C) 2002-2008 Novamente LLC
 * All Rights Reserved
 *
 * Written by Nil Geisweiller
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#include "ant_action_symbol.h"
#include <moses3/comboreduct/type_checker/type_tree.h>

namespace moses3 { namespace combo {

using namespace ant_action_symbol_properties;

ant_action_symbol::ant_action_symbol() { }

const action_symbol_basic_description* ant_action_symbol::get_basic_description_array() const {
  return asbd;
}

unsigned int ant_action_symbol::get_basic_description_array_count() const {
  return sizeof(asbd)/sizeof(basic_description);
}

const ant_action_symbol* ant_action_symbol::init_action_symbol() {
  static ant_action_symbol* action_symbols =
    new ant_action_symbol[id::ant_action_symbol_count];
  for(unsigned int i = 0; i < id::ant_action_symbol_count; ++i)
    action_symbols[i].set_action_symbol((ant_action_symbol_enum)i);
  return action_symbols;
}

void ant_action_symbol::set_action_symbol(ant_action_symbol_enum pase) {
  OC_ASSERT(pase<id::ant_action_symbol_count);
  _enum = pase;
  //fill the various properties using the arrays edited by the developer
  set_basic_description(pase);
}

action_symbol ant_action_symbol::get_instance(const std::string& name) {
  //look up for pet_action_symbol_enum corresponding to that name
  bool found = false;
  action_symbol as = NULL;
  for(unsigned int i = 0; i<id::ant_action_symbol_count && !found; ++i) {
    as = ant_action_symbol::get_instance((ant_action_symbol_enum)i);
    found = as->get_name()==name;
  }
  return (found? as : NULL);
}

action_symbol ant_action_symbol::get_instance(ant_action_symbol_enum pase) {
  static const ant_action_symbol* action_symbols=init_action_symbol();
  OC_ASSERT(pase<id::ant_action_symbol_count);
  return static_cast<action_symbol>(&action_symbols[pase]);
}

const std::string& ant_action_symbol::get_name() const {
  return _name;
}

const type_tree& ant_action_symbol::get_type_tree() const {
  return _type_tree;
}

arity_t ant_action_symbol::arity() const {
  return _arity;
}

type_tree ant_action_symbol::get_output_type_tree() const {
  return _output_type;
}

const type_tree& ant_action_symbol::get_input_type_tree(arity_t i) const {
  return argument_type_list_input_type(_arg_type_tree, _arity, i);
}

}} // ~namespaces combo moses3
