/*
 * moses3/comboreduct/combo/message.cc
 *
 * Copyright (C) 2002-2008, 2012 Novamente LLC
 * All Rights Reserved
 *
 * Written by Nil Geisweiller, Linas Vepstas
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#include "enum_type.h"
#include <moses3/util/mt19937ar.h>

namespace moses3 { namespace combo {
using namespace std;

// Global table of string-to-int, so that the operator==() can run
// efficiently (for scoring) without having to do a string-compare.
unsigned enum_t::enum_issued = 0;
map<string, unsigned> enum_t::enum_map;
boost::shared_mutex enum_t::id_mutex;

enum_t enum_t::get_random_enum()
{
    int r = randGen().randint(enum_issued);

    map<string, unsigned>::iterator entry = enum_map.begin();
    while (r) {
       r--; entry ++;
    }
    return enum_t(entry->first, entry->second);
}

unsigned enum_t::get_id(const string& token)
{
    typedef boost::shared_mutex mutex;
    typedef boost::unique_lock<mutex> unique_lock;

    unique_lock lock(id_mutex);

    map<string, unsigned>::iterator entry = enum_map.find(token);
    if (entry == enum_map.end()) {
        enum_issued ++;
        entry = enum_map.insert(pair<string, unsigned>(token, enum_issued)).first;
    }
    return (unsigned) entry->second;
}

size_t enum_t::size()
{
    return enum_map.size();
}

ostream& operator<<(ostream& out, const combo::enum_t& m)
{
    return out << combo::enum_t::prefix()
               << "(" << m.getId() << "):"
               << '\"' << m.getContent() << '\"';
}

} // ~namespace combo
} // ~namespace moses3

