/*
 * moses3/comboreduct/combo/message.h
 *
 * Copyright (C) 2002-2008 Novamente LLC
 * All Rights Reserved
 *
 * Written by Nil Geisweiller
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#ifndef _COMBO_MESSAGE_H
#define _COMBO_MESSAGE_H

#include <ostream>
#include <set>
#include <string>
#include <boost/operators.hpp>

namespace moses3 { namespace combo {

// message is essentially a string but is coded as a different type
// than definite_object, because it semantically denotes something else.
class message
    : boost::less_than_comparable<message>, // generate >, <= and >= given <
      boost::equality_comparable<message>   // generate != given ==
{
private:
    std::string _content;
public:
    message(const std::string& m) {
        _content = m;
    }

    std::string getContent() const {
        return _content;
    }

    /// @todo: this operator is used for scoring, and so the string
    /// compare may make things slow... perhaps we should use a map
    /// the same way that enum_t does.
    bool operator==(const message& m) const {
        return _content == m.getContent();
    }

    bool operator<(const message& m) const {
        return _content < m.getContent();
    }
    
    /// This is used by message_str_to_vertex() to identify a message.
    static std::string prefix() {
        return "message:";
    }
};

typedef std::set<message> message_set;
typedef message_set::iterator message_set_it;
typedef message_set::const_iterator message_set_const_it;

std::ostream& operator<<(std::ostream&, const moses3::combo::message&);

} // ~namespace combo
} // ~namespace moses3

#endif

