/*
 * moses3/comboreduct/combo/action_eval.h
 *
 * Copyright (C) 2002-2008 Novamente LLC
 * All Rights Reserved
 *
 * Written by Nil Geisweiller
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#ifndef _COMBO_ACTION_EVAL_H
#define _COMBO_ACTION_EVAL_H

#include "util/exceptions.h"
#include "util/tree.h"

#include "comboreduct/combo/vertex.h"
#include "comboreduct/crutil/exception.h"
#include "comboreduct/combo/using.h"
#include "comboreduct/combo/type_tree.h"

#include <exception>

namespace moses3 { namespace combo {

template<typename It>
vertex action_eval_throws(It it) throw(EvalException, AssertionException, std::bad_exception) {
    typedef typename It::sibling_iterator sib_it;
    //argument
    //WARNING : we assume the argument is necessarily an action and therefor
    //never has a negative index
    if(is_argument(*it)) {
        arity_t idx=get_argument(*it).idx;
        //assumption : when idx is negative the argument is necessary boolean
        OC_ASSERT(idx > 0, "argument is necessarily an action and therefore never has a neg idx.");
        if (const vertex* v=boost::get<const vertex>(&binding(idx)))
            return *v;
        else
            return action_eval_throws(boost::get<combo_tree::iterator>(binding(idx)));
    }
    //action sequence
    if(*it==id::sequential_and) {
        for(sib_it sib = it.begin(); sib != it.end(); ++sib) {
            if(action_eval_throws(sib) != id::action_success)
                return id::action_failure;
        }
        return id::action_success;
    }
    else if(*it==id::sequential_or) {
        for(sib_it sib = it.begin(); sib != it.end(); ++sib) {
            if(action_eval_throws(sib) == id::action_success)
                return id::action_success;
        }
        return id::action_failure;
    }
    else if(*it==id::sequential_exec) {
        for(sib_it sib = it.begin(); sib != it.end(); ++sib)
            action_eval_throws(sib);
        return id::action_success;
    }
    //action_if
    else if(*it==id::boolean_action_if) {
        OC_ASSERT(it.number_of_children()==3, 
                  "combo_tree node should have exactly three children (id::boolean_if).");
        sib_it sib = it.begin();
        vertex vcond = action_eval_throws(sib);
        ++sib;
        if(vcond==id::action_success) {
            //TODO : check that b1 and b2 have type boolean
            return eval_throws(sib);
        }
        else {
            ++sib;
            return eval_throws(sib);
        }
    }
    else if(*it==id::contin_action_if) {
        OC_ASSERT(it.number_of_children()==3,
                  "combo_tree node should have exactly three children (id::contin_action_if).");
        sib_it sib = it.begin();
        vertex vcond = action_eval_throws(sib);
        ++sib;
        if(vcond==id::action_success) {
            //TODO : check that b1 and b2 have type contin
            return eval_throws(sib);
        }
        else {
            ++sib;
            return eval_throws(sib);
        }
    }
    else if(*it==id::action_action_if) {
        OC_ASSERT(it.number_of_children()==3,
                  "combo_tree node should have exactly three children (id::contin_action_if).");
        sib_it sib = it.begin();
        vertex vcond = action_eval_throws(sib);
        ++sib;
        if(vcond==id::action_success) {
            //TODO : check that b1 and b2 have type action
            return action_eval_throws(sib);
        }
        else {
            ++sib;
            return action_eval_throws(sib);
        }
    }
    else {
        return *it;
    }
}

} // ~namespace combo
} // ~namespace moses3 

#endif
