/*
 * moses3/comboreduct/main/contin-reductor.cc
 *
 * Copyright (C) 2002-2008 Novamente LLC
 * All Rights Reserved
 *
 * Written by Nil Geisweiller
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#include <iostream>

#include <moses3/util/mt19937ar.h>

#include "../reduct/reduct.h"
#include "../interpreter/eval.h"
#include "../table/table.h"
#include "../table/table_io.h"

using namespace std;
using namespace moses3;
using namespace reduct;

int main()
{
    randGen().seed(1);
    int reduct_effort = 2;
    combo_tree tr;

    while (cin.good()) {
        cin >> tr;
        if (!cin.good())
            break;

        type_tree tt = infer_type_tree(tr);

        //produce random inputs
        ITable cti(tt);

        try {

            //print cti, for debugging
            cout << "Rnd matrix :" << endl << cti;


            //evalutate tr over cti and fill ct1
            OTable ct1(tr, cti);

            //print the tree before reduction, for debugging
            cout << "Before : " << tr << endl;

            contin_reduce(tr, reduct_effort, vertex_set());

            //evaluate tr over cti and fill ct2
            OTable ct2(tr, cti);

            cout << "After  : " << tr << endl;
            if (ct1 != ct2) {
                cout << ct1 << endl << ct2 << endl;
                cerr << "contin tables don't match!" << endl;
                //return 1;
            }
        } catch (EvalException& e) {
            cout << e.get_message() << " : " << e.get_vertex() << endl;
        }
    }
    return 0;
}
