/*
 * moses3/comboreduct/main/full-reductor.cc
 *
 * Copyright (C) 2002-2008 Novamente LLC
 * All Rights Reserved
 *
 * Written by Nil Geisweiller
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#include <iostream>

#include <moses3/util/mt19937ar.h>

#include "../reduct/reduct.h"
#include "../interpreter/eval.h"
#include "../table/table.h"
#include "../table/table_io.h"
#include "../type_checker/type_tree.h"

using namespace std;
using namespace moses3;
using namespace reduct;

int main()
{
    randGen().seed(1);

    combo_tree tr;

    while (cin.good()) {
        cin >> tr;
        if (!cin.good())
            break;

        // determine the type of tr
        type_tree tt = infer_type_tree(tr);
        cout << "Type : " << tt << endl;

        bool ct = is_well_formed(tt);

        if (!ct) {
            cout << "Bad type" << endl;
            break;
        }

        // produce random inputs
        ITable cti(tt);
        // print cti, for debugging
        cout << "Rnd matrix :" << endl << cti;

        try {
            // evalutate tr over cti and fill mt1
            OTable mt1(tr, cti);
            //print mt1, for debugging
            cout << "MT1" << endl << mt1 << endl;

            //print the tree before reduction, for debugging
            cout << "Before : " << tr << endl;

            full_reduce(tr);

            // evaluate tr over cti and fill mt2
            OTable mt2(tr, cti);
            //print mt2, for debugging
            cout << "MT2" << endl << mt2 << endl;

            cout << "After  : " << tr << endl;
            if (mt1 != mt2) {
                cout << mt1 << endl << mt2 << endl;
                cerr << "mixed tables don't match!" << endl;
                return 1;
            }
        } catch (EvalException& e) {
            cout << e.get_message() << " : " << e.get_vertex() << endl;
        }
    }
    return 0;
}
