/*
 * moses3/comboreduct/main/perception-reduct.cc
 *
 * Copyright (C) 2002-2008 Novamente LLC
 * All Rights Reserved
 *
 * Written by Nil Geisweiller
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#include <iostream>

#include <moses3/util/mt19937ar.h>

#include "../combo/vertex.h"
#include "../reduct/reduct.h"
#include "../interpreter/eval.h"
#include "../table/table.h"
#include "../table/table_io.h"
#include "../type_checker/type_tree.h"

using namespace std;
using namespace moses3;
using namespace reduct;

int main()
{

    combo_tree tr;

    randGen().seed(0);

    while (cin.good()) {
        cin >> tr;
        if (!cin.good())
            break;

        //get type_node tree of tr.
        type_tree tt = infer_type_tree(tr);
        cout << "Type : " << tt << endl;

        //check whether the tree is well-formed.
        bool ct = is_well_formed(tt);

        if (!ct) {
            cout << "Bad type" << endl;
            break;
        }

        //produce random inputs
        ITable cti(tt);
        //print cti, for debugging
        cout << "Rnd matrix :" << endl << cti;

        try {
            //evalutate tr over cti and fill mat1
            //mixed_action_table mat1(tr, cti, tr_type);
            //print mat1, for debugging
            //cout << "MAT1" << endl << mat1 << endl;

            //print the tree before reduction, for debugging
            cout << "Before : " << tr << endl;

            perception_reduce(tr);

            //evaluate tr over cti and fill mat2
            //mixed_action_table mat2(tr, cti, tr_type);
            //print mat2, for debugging
            //cout << "MAT2" << endl << mat2 << endl;

            cout << "After  : " << tr << endl;
            //if (mat1!=mat2) {
            //cout << mat1 << endl << mat2 << endl;
            //cerr << "mixed-perception-tables don't match!" << endl;
            //return 1;
            //}
        } catch (EvalException& e) {
            cout << e.get_message() << " : " << e.get_vertex() << endl;
        }
    }
    return 0;
}
