/** algo/incremental.cc ---
 *
 * Copyright (C) 2011 OpenCog Foundation
 *
 * Author: Nil Geisweiller
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <boost/range/irange.hpp>

#include "incremental.h"

namespace moses3 {

using namespace std;

feature_set incremental_select_features(const CTable& ctable,
                                        const feature_selection_parameters& fs_params)
{
    auto ir = boost::irange(0, ctable.get_arity());
    feature_set all_features(ir.begin(), ir.end());

    logger().info() << "incremental_select_features, targ sz="
                    << fs_params.target_size
                    << " thresh=" << fs_params.threshold;
    if (fs_params.threshold <= 0 && fs_params.target_size <= 0) {
        // Nothing happened, return all features by default
        return all_features;
    }

    typedef MutualInformation<feature_set> FeatureScorer;
    FeatureScorer fsc(ctable);
    if (0 < fs_params.target_size) {
        return cached_adaptive_incremental_selection(all_features, fsc,
                                              fs_params.target_size,
                                              fs_params.inc_interaction_terms,
                                              fs_params.inc_red_intensity,
                                              0, 1,
                                              fs_params.inc_target_size_epsilon);
    }
    return cached_incremental_selection(all_features, fsc,
                                   fs_params.threshold,
                                   fs_params.inc_interaction_terms,
                                   fs_params.inc_red_intensity);
}

} // ~namespace moses3
