/** algo/simple.cc ---
 *
 * Copyright (C) 2011 OpenCog Foundation
 * Copyright (C) 2012 Poulin Holdings LLC
 *
 * Author: Nil Geisweiller
 * Author: Linas Vepstas <linasvepstas@gmail.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <boost/range/irange.hpp>

#include "simple.h"

namespace moses3 {
    
using namespace std;

feature_set simple_select_features(const CTable& ctable,
                                   const feature_selection_parameters& fs_params)
{
    auto ir = boost::irange(0, ctable.get_arity());
    feature_set all_features(ir.begin(), ir.end());
    if (0 == fs_params.target_size)
        // Nothing happened, return all features by default
        return all_features;
        
    typedef MutualInformation<feature_set> FeatureScorer;
    FeatureScorer fsc(ctable);
    return simple_selection(all_features, fsc,
                            fs_params.target_size,
                            fs_params.threshold,
                            fs_params.inc_red_intensity);
}

} // ~namespace moses3
