/** 
 * simple.h ---
 *
 * Copyright (C) 2012 Poulin Holdings LLC
 *
 * Author: Linas Vepstas <linasvepstas@gmail.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#ifndef _OPENCOG_FEATURE_SELECTION_SIMPLE_ALGO_H
#define _OPENCOG_FEATURE_SELECTION_SIMPLE_ALGO_H

#include <moses3/util/numeric.h>

#include "../main/feature-selection.h"  // needed for feature_set, feature_selection_parameters

namespace moses3 {

feature_set simple_select_features(const CTable& ctable,
                                   const feature_selection_parameters& fs_params);

/**
 * Returns a set S of features following the algo:
 *
 * @param features       The initial set of features to be selected from
 * @param scorer         The function to score a set of features.
 * @param threshold      The threshold to select a set of feature
 * @param red_threshold  If >0 it modulates the intensity of the
 *                       threshold of redundant_features(), precisely
 *                       red_threshold * threshold
 *                       Otherwise redundant features are ignored.
 *
 * @return               The set of selected features
 */
template<typename Scorer, typename FeatureSet>
FeatureSet simple_selection(const FeatureSet& features,
                            const Scorer& scorer,
                            int num_desired,
                            double threshold,
                            double red_threshold = 0)
{
    std::multimap<double, FeatureSet> sorted_flist;

    for (auto feat : features) {
        FeatureSet fs{feat};
        double sc = scorer(fs);
        if (threshold <= sc) {
            sorted_flist.insert({sc, fs});
        }
    }

    FeatureSet final;
    // for (auto pr : sorted_flist) {
    for (auto pr = sorted_flist.rbegin(); pr != sorted_flist.rend(); pr++) {
        // std::cout << "sc="<< pr->first <<" fe=" << *pr->second.begin() << std::endl;
        final.insert(*pr->second.begin());
        num_desired --;
        if (num_desired <= 0) break;
    }

    // TODO: if there is a non-negative red_threshold, then remove any
    // pair-wise redundant features. 

    return final;
}


} // ~namespace moses3

#endif // _OPENCOG_FEATURE_SELECTION_SIMPLE_ALGO_H
