/** algo/stochastic_max_dependency.cc ---
 *
 * Copyright (C) 2011 OpenCog Foundation
 * Copyright (C) 2012 Poulin Holdings LLC
 *
 * Author: Linas Vepstas <linasvepstas@gmail.com
 *         Nil Geisweiller
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <boost/range/irange.hpp>

#include "stochastic_max_dependency.h"

namespace moses3 {

using namespace std;

/// Select features, using the SMD algorithm
feature_set smd_select_features(const CTable& ctable,
                                const feature_selection_parameters& fs_params)
{
    auto ir = boost::irange(0, ctable.get_arity());

    feature_set all_features(ir.begin(), ir.end());

    if (fs_params.target_size <= 0)
        // Nothing happened, return the all features by default
        return all_features;

    feature_set init_features = initial_features(ctable.get_input_labels(), fs_params);

    fs_scorer<set<arity_t> > fs_sc(ctable, fs_params);
    return stochastic_max_dependency_selection(all_features, init_features,
                                               fs_sc,
                                               (unsigned) fs_params.target_size,
                                               fs_params.threshold,
                                               fs_params.smd_top_size);
}


} // ~namespace moses3
