/** scorers/moses_matrix.h --- 
 *
 * Copyright (C) 2011 OpenCog Foundation
 *
 * Author: Nil Geisweiller <nilg@desktop>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#ifndef _OPENCOG_FS_SCORERS_MATRIX_H
#define _OPENCOG_FS_SCORERS_MATRIX_H

#include <moses3/util/numeric.h>

#include <moses3/learning/moses/moses/scoring.h>
#include <moses3/comboreduct/combo/common_def.h>

namespace moses3 {

using namespace moses;
using namespace combo;

/**
 * Wrapper to use moses scoring precision (see
 * moses3/learning/moses/moses/scoring.h).  This is one of the
 * confusion-matrix based scorers.
 *
 * That wrapper uses the method best_possible_score() given a certain
 * feature set. And therefore attempts to maximize the best possible
 * score one would get (w.r.t. some fitness function) given the
 * feature set being evaluated.
 */
template<typename FeatureSet>
struct pre_scorer : public unary_function<FeatureSet, double>
{
    pre_scorer(const CTable& ctable,
               float penalty = 1.0f,
               float min_activation = 0.5f,
               float max_activation = 1.0f,
               bool positive = true)
        : _ctable(ctable), _penalty(penalty),
          _min_activation(min_activation), _max_activation(max_activation),
          _positive(positive) {}

    double operator()(const FeatureSet& fs) const {
        // filter the ctable
        CTable filtered_ctable = _ctable.filtered(fs);
        // create the scorer
        precision_bscore sc(filtered_ctable, _penalty,
                            _min_activation, _max_activation, _positive);
        return boost::accumulate(sc.best_possible_bscore(), 0.0);
    }
protected:
    const CTable& _ctable;
    float _penalty, _min_activation, _max_activation;
    bool _positive;
};


} // ~namespace moses3

#endif // _OPENCOG_FS_SCORERS_MATRIX_H
