/** scorers/moses_optim.h --- 
 *
 * Copyright (C) 2011 OpenCog Foundation
 *
 * Author: Nil Geisweiller <nilg@desktop>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#ifndef _OPENCOG_FS_SCORERS_OPTIM_H
#define _OPENCOG_FS_SCORERS_OPTIM_H

#include <moses3/util/numeric.h>

#include <moses3/learning/moses/representation/field_set.h>
#include <moses3/learning/moses/eda/eda.h>
#include <moses3/learning/moses/moses/types.h>
#include <moses3/learning/moses/moses/scoring.h>
#include <moses3/comboreduct/combo/common_def.h>

namespace moses3 {

using namespace moses;
using namespace combo;

/**
 * translate an instance into a feature set. Each feature is
 * represented by its index (the left most one is 0).
 */
std::set<arity_t> get_feature_set(const field_set& fields,
                                  const instance& inst);

/**
 * Wrapper to use a feature set scorer with MOSES's optimization
 * algorithms operating on a deme. Each deme is a binary string where
 * each bit represents whether a feature is selected or not.
 */
template<typename FSScorer>
struct deme_based_scorer : public iscorer_base
{
    deme_based_scorer(const FSScorer& fs_scorer, const field_set& fields)
        : _fs_scorer(fs_scorer), _fields(fields) {}

    /**
     * The feature set is represented by an instance encoding a
     * field of booleans. Each boolean represents whether its
     * corresponding feature is in the feature set of not.
     */
    composite_score operator()(const instance& inst) const
    {
        std::set<arity_t> fs = get_feature_set(_fields, inst);
        composite_score csc(_fs_scorer(fs), fs.size(), 0);
        // Logger
        if (logger().isFineEnabled()) {
            logger().fine()
               << "moses_based_scorer - Evaluate instance: " 
               << _fields.stream(inst) << " " << csc;
        }
        // ~Logger
        return csc;
    }

    const FSScorer& _fs_scorer;
    const field_set& _fields;
};

} // ~namespace moses3

#endif // _OPENCOG_FS_SCORERS_OPTIM_H
