/*
 * moses3/learning/moses/eda/initialization.cc
 *
 * Copyright (C) 2002-2008 Novamente LLC
 * All Rights Reserved
 *
 * Written by Moshe Looks
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#include "initialization.h"
#include "../moses/neighborhood_sampling.h"

namespace moses3 { 
namespace moses {

using namespace std;

void occam_randomize_contin(const field_set& fs, instance& inst,
                            field_set::contin_iterator it)
{
    unsigned int n = randGen().randint(fs.contin()[it.idx()].depth);
    moses::generate_contin_neighbor(fs, inst, it, n);
}

//tree should be roughly balanced for this to be effective - otherwise there
//will be a bias towards smaller programs
void occam_randomize_term(const field_set& fs, instance& inst,
                          field_set::const_term_iterator it)
{
    //if there are n levels
    size_t begin = fs.term_to_raw_idx(it.idx());
    size_t end = begin + fs.term()[it.idx()].depth;
    size_t middle = begin + randGen().randint(end - begin + 1);

    const term_tree& tr = (*fs.term()[it.idx()].tr);
    term_tree::iterator node = tr.begin();

    for (;begin != middle && !node.is_childless();++begin) {
        int child_idx = randGen().randint(node.number_of_children());
        fs.set_raw(inst, begin,
                   field_set::term_spec::from_child_idx(child_idx));
        node = tr.child(node, child_idx);
    }
    for (;begin != end;++begin)
        fs.set_raw(inst, begin, field_set::term_spec::Stop);
}

void occam_randomize_term(const field_set& fs, instance& inst)
{
    for (field_set::const_term_iterator it = fs.begin_term(inst);
            it != fs.end_term(inst);++it)
        occam_randomize_term(fs, inst, it);
}

void occam_randomize_contin(const field_set& fs, instance& inst)
{
    for (field_set::contin_iterator it = fs.begin_contin(inst);
         it != fs.end_contin(inst);++it)
        occam_randomize_contin(fs, inst, it);
}

void uniform_randomize_bit(const field_set& fs, instance& inst)
{
    //could be faster
    generate(fs.begin_bit(inst), fs.end_bit(inst),
             bind(&RandGen::randbool, ref(randGen())));
}

void uniform_randomize_disc(const field_set& fs, instance& inst)
{
    for (field_set::disc_iterator it = fs.begin_disc(inst);
            it != fs.end_disc(inst);++it)
        it.randomize();
}

void randomize(const field_set& fs, instance& inst)
{
    occam_randomize_term(fs, inst);
    occam_randomize_contin(fs, inst);
    uniform_randomize_disc(fs, inst);
    uniform_randomize_bit(fs, inst);
}

} // ~namespace moses
} // ~namespace moses3

