/*
 * moses3/learning/moses/moses/ant_knobs.h
 *
 * Copyright (C) 2002-2008 Novamente LLC
 * All Rights Reserved
 *
 * Written by Moshe Looks
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#ifndef _MOSES_ANT_KNOBS_H
#define _MOSES_ANT_KNOBS_H

#include <moses3/comboreduct/ant_combo_vocabulary/ant_combo_vocabulary.h>
#include "knobs.h"

namespace moses3 { namespace moses {

using namespace ant_combo;

// Knob specifically designed for the Sante Fe Trail problem.
// Note - children aren't cannonized when parents are called.
struct ant_action_subtree_knob : public discrete_knob<4> {
    static const int none    = 0;
    static const int forward = 1;
    static const int rleft   = 2;   // rotate left
    static const int rright  = 3;   // rotate right

    ant_action_subtree_knob(combo_tree& tr, combo_tree::iterator tgt,
                            combo_tree::iterator subtree)
        : discrete_knob<4>(tr) {

        _default = none;
        _current = _default;

        _loc = _tr.append_child(tgt, id::null_vertex);
        _tr.append_child(_loc, subtree);
    }

    complexity_t complexity_bound() const {
        return tree_complexity(_loc);
    }

    void clear_exemplar() {
        if (in_exemplar())
            turn(0);
        else
            _tr.erase(_loc);
    }

    void turn(int idx) {
        idx = map_idx(idx);
        OC_ASSERT((idx < 4), "Index greater than 3.");

        if (idx == _current) //already set, nothing to
            return;

        if (idx == none) {
            if (_current != none)
                _loc = _tr.insert_above(_loc, id::null_vertex);
        } else {
            if (_current == none)
                _tr.erase_children(_loc);

            switch (idx) {
            case forward:
                *_loc = get_instance(id::move_forward);
                break;

            case rleft:
                *_loc = get_instance(id::turn_left);
                break;

            case rright:
                *_loc = get_instance(id::turn_right);
                break;

            default:
                break;
            }
        }

        _current = idx;
    }

    field_set::disc_spec spec() const {
        return field_set::disc_spec(multiplicity());
    }

    std::string toStr() const {
        stringstream ss;
        ss << "[" << *_loc << " TODO ]";
        return ss.str();
    }
};

} //~namespace moses
} //~namespace moses3

#endif

