/*
 * moses3/learning/moses/main/moses-ant-hillclimbing.cc
 *
 * Copyright (C) 2002-2008 Novamente LLC
 * All Rights Reserved
 *
 * Written by Predrag Janicic
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#include <algorithm>
#include <iostream>
#include <set>
#include <string>
#include <boost/lexical_cast.hpp>

#include <moses3/util/mt19937ar.h>
#include <moses3/util/numeric.h>

#include <moses3/comboreduct/combo/vertex.h>
#include <moses3/comboreduct/ant_combo_vocabulary/ant_combo_vocabulary.h>
#include <moses3/comboreduct/reduct/reduct.h>

#include "../moses/moses_main.h"
#include "../optimization/optimization.h"
#include "ant_scoring.h"


using namespace moses3;
using namespace moses;
using namespace reduct;
using namespace boost;
using namespace ant_combo;
using namespace std;

int main(int argc,char** argv) 
{
  int max_evals,rand_seed;
  try {
    if (argc!=3)
      throw "foo";
    rand_seed=lexical_cast<int>(argv[1]);
    randGen().seed(rand_seed);
    max_evals=atoi(argv[2]);
  } catch (...) {
    cerr << "usage: " << argv[0] << " seed maxevals" << endl;
    exit(1);
  }

  type_tree tt(id::lambda_type);
  tt.append_children(tt.begin(),id::action_result_type,1);

  ant_score scorer;
  ant_bscore bscorer;

  combo_tree_ns_set perceptions;
  combo_tree_ns_set actions;

  actions.insert(combo_tree(get_instance(id::turn_left)));
  actions.insert(combo_tree(get_instance(id::turn_right)));
  actions.insert(combo_tree(get_instance(id::move_forward)));

  perceptions.insert(combo_tree(get_instance(id::is_food_ahead)));

  metapop_parameters metaparms;
  metaparms.perceptions = &perceptions;
  metaparms.actions = &actions;

  hill_climbing hc;
  metapopulation metapop(combo_tree(id::sequential_and), tt, action_reduction(),
              scorer, bscorer, hc, metaparms);
  
  boost::program_options::variables_map vm;
  jobs_t jobs;

  moses_parameters moses_param(vm, jobs, true, max_evals, -1, 0);
  moses_statistics st;
  run_moses(metapop, moses_param, st);
}

