/** moses-ann-pole1.cc --- 
 *
 * Copyright (C) 2010-2011 OpenCog Foundation
 *
 * Author: Joel Lehman
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#include <iostream>
#include <moses3/util/mt19937ar.h>
#include <moses3/util/Logger.h>
#include <moses3/comboreduct/interpreter/eval.h>

#include "../moses/moses_main.h"
#include "../moses/scoring.h"
#include "../moses/ann_scoring.h"
#include "../representation/representation.h"
#include "../optimization/optimization.h"

using namespace std;
using namespace boost;
using namespace moses3;
using namespace moses;
using namespace reduct;

int main(int argc, char** argv)
{

    // Set flag to print only cassert and other ERROR level logs on stdout.
    logger().setPrintErrorLevelStdout();

    // Read in maximum evaluations and RNG seed from command line.
    int seed;
    bool reduce=true;
    try
    {
        //if (argc!=3)
        //    throw "foo";
        // int max_evals=lexical_cast<int>(argv[1]);
        seed=lexical_cast<int>(argv[2]);
        
        set_stepsize(lexical_cast<double>(argv[3]));
        set_expansion(lexical_cast<double>(argv[4]));
        set_depth(lexical_cast<int>(argv[5]));

        reduce = lexical_cast<int>(argv[6]);
    }
    catch (...)
    {
        cerr << "usage: " << argv[0] << " maxevals seed" << endl;
        exit(1);
    }
    
    // Read seed tree in from stdin.
    combo_tree tr;
    cin >> tr; 

    randGen().seed(seed);

    type_tree tt(id::lambda_type);
    tt.append_children(tt.begin(), id::ann_type, 1);
    
    const reduct::rule* si = &(ann_reduction());
    if(!reduce)
        si = &(clean_reduction());

    //SINGLE MARKOVIAN POLE TASK
    ann_pole_cscore p_cscore;
    ann_pole_bscore p_bscore; 

    univariate_optimization optim_algo;
    metapopulation metapop_pole(tr, tt, *si, p_cscore, p_bscore, optim_algo);

    moses_parameters pa;
    moses_statistics st;
    run_moses(metapop_pole, pa, st);

    //change best tree into ANN
    tree_transform trans; 
    combo_tree best = metapop_pole.best_tree();
    ann bestnet = trans.decodify_tree(best);
   
    //write out the best network
    cout << "Best network: " << endl; 
    cout << &bestnet << endl;

    //write out in dot format
    bestnet.write_dot("best_nn.dot");
    
    //for parameter sweep
    cout << metapop_pole.best_score() << endl;
}
