/** eval-diversity.h --- 
 *
 * Copyright (C) 2012 OpenCog Foundation
 *
 * Author: Nil Geisweiller <ngeiswei@gmail.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <vector>

#include <boost/range/algorithm/transform.hpp>
#include <boost/accumulators/accumulators.hpp>
#include <boost/accumulators/statistics/stats.hpp>
#include <boost/accumulators/statistics/count.hpp>
#include <boost/accumulators/statistics/mean.hpp>
#include <boost/accumulators/statistics/variance.hpp>
#include <boost/accumulators/statistics/min.hpp>
#include <boost/accumulators/statistics/max.hpp>

#include <moses3/util/iostreamContainer.h>

namespace moses3 { namespace moses {

using namespace boost::accumulators;

// structure holding the options
struct eval_diversity_params {
    // IO
    std::vector<std::string> input_files,
        moses_files;
    std::string output_file,
        target_feature;
    bool display_stats,
        display_values;
    // parameters
    std::string diversity_dst;
    float diversity_p_norm;
};

// diversity distance types
static const std::string p_norm = "p_norm";
static const std::string tanimoto = "tanimoto";
static const std::string angular = "angular";
        
// define accumulator to gather stats
typedef accumulator_set<float, stats<tag::count,
                                     tag::mean,
                                     tag::variance,
                                     tag::min,
                                     tag::max>> accumulator_t;

template<typename Out>
Out& ostream_results(Out& out, const eval_diversity_params& edp,
                     const std::vector<float>& dsts) {
    if (edp.display_values)
        ostreamContainer(out, dsts, "\n") << std::endl;

    if (edp.display_stats) {
        // compute the statistics
        accumulator_t acc;
        for (float f : dsts) acc(f);

        // display the statistics
        out << "count: " << count(acc) << std::endl;
        out << "mean: " << mean(acc) << std::endl;
        out << "std dev: " << sqrt(variance(acc)) << std::endl;
        out << "min: " << boost::accumulators::min(acc) << std::endl;
        out << "max: " << boost::accumulators::max(acc) << std::endl;
    }
    return out;
}

} // ~namespace moses
} // ~namespace moses3

