/*
 * moses3/learning/moses/moses/feature_selector.cc
 *
 * Copyright (C) 2012 OpenCog Foundation
 * All Rights Reserved
 *
 * Written by Nil Geisweiller
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <moses3/comboreduct/table/table.h>
#include <moses3/comboreduct/combo/vertex.h>
#include <moses3/learning/feature-selection/main/feature-selection.h>

namespace moses3 {

/**
 * Struct in charge of selecting features maximize the amount of
 * information in combination with a candidate.
 */
struct feature_selector
{
    typedef std::set<combo::arity_t> feature_set;

    feature_selector(const combo::CTable& ctable,
                     const feature_selection_parameters& fs_params)
        : _ctable(ctable), _fs_params(fs_params)
    {}

    feature_selector(const combo::Table& table,
                     const feature_selection_parameters& fs_params)
        : _ctable(table.compressed()), _fs_params(fs_params)
    {}

    /// Return a feature set that correlates well with all the rows
    /// where the combo tree is predicting an incorrect result.
    /// The hope is that these will be used to build a more accurate
    /// models.
    feature_set operator()(const combo::combo_tree& tr) const
    {
        combo::CTable activ_ctable(_ctable.get_labels(), _ctable.get_signature());
        // Use only rows where the model is just plain wrong.  The
        // feature selector will then look for any features that
        // correlate well with these incorrect answers.
        for (const combo::CTable::value_type& vct : _ctable) {
            vertex predicted_out = eval_binding(vct.first, tr);
            Counter<vertex, unsigned> cnt = vct.second;
            vertex actual_out = cnt.most_frequent();
            if (predicted_out != actual_out)
                activ_ctable.insert(vct);
        }
        // Call feature selection
        return select_features(activ_ctable, _fs_params);
        // return feature_set();
    }

    const combo::CTable& _ctable;
    feature_selection_parameters _fs_params;
};

} // ~namespace moses3
