/*
 * moses3/learning/moses/moses/knobs.cc
 *
 * Copyright (C) 2002-2008 Novamente LLC
 * All Rights Reserved
 *
 * Written by Moshe Looks
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#include "knobs.h"

namespace moses3 { namespace moses {

std::string logical_subtree_knob::toStr() const
{
    std::stringstream ss;
    ss << "[";
    for(int i = 0; i < multiplicity(); ++i)
        ss << posStr(map_idx(i)) << (i < multiplicity()-1? " " : "");
    ss << "]";
    return ss.str();
}

// return << *_loc or << *_loc.begin() if it is null_vertex
// if *_loc is a negative literal returns !$n
// if negated is true a copy of the literal is negated before being printed
std::string logical_subtree_knob::locStr(bool negated) const
{
    OC_ASSERT(*_loc != id::null_vertex || _loc.has_one_child(),
              "if _loc is null_vertex then it must have only one child");
    std::stringstream ss;
    combo_tree::iterator it;
    if (*_loc == id::null_vertex)
        it = _loc.begin();
    else it = _loc;
    if (is_argument(*it)) {
        argument arg = get_argument(*it);
        if (negated) arg.negate();
        ostream_abbreviate_literal(ss, arg);
    } else {
        ss << (negated? "!" : "") << *it;
    }
    return ss.str();
}

// Return the name of the position, if it is the current one and
// tag_current is true then the name is put in parenthesis.
std::string logical_subtree_knob::posStr(int pos, bool tag_current) const
{
    std::stringstream ss;
    switch (pos) {
    case absent:
        ss << "nil";
        break;
    case present:
        ss << locStr();
        break;
    case negated:
        ss << locStr(true);
        break;
    default:
        ss << "INVALID SETTING";
    }
    return pos == _current && tag_current?
        std::string("(") + ss.str() + ")" : ss.str();
}

} //~namespace moses
} //~namespace moses3
