/*
 * moses3/util/hashing.h
 *
 * Copyright (C) 2002-2007 Novamente LLC
 * All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef _OPENCOG_HASHING_H
#define _OPENCOG_HASHING_H

#include <boost/functional/hash.hpp>

#include <moses3/util/tree.h>

namespace moses3
{

// Functor returning the address of an object pointed by an
// iterator. Useful for defining the hash function of an iterator.
template<typename It>
struct obj_ptr_hash {
    size_t operator()(const It& it) const {
        return boost::hash_value(&(*it));
    }
};

template < typename T,
typename Hash = boost::hash<T> >
struct deref_hash {
    deref_hash(const Hash& h = Hash()) : hash(h) {}
    size_t operator()(const T& t) const {
        return hash(*t);
    }
    Hash hash;
};

template < typename T,
typename Equals = std::equal_to<T> >
struct deref_equals {
    deref_equals(const Equals& e = Equals()) : equals(e) {}
    bool operator()(const T& x, const T& y) const {
        return equals(*x, *y);
    }
    Equals equals;
};

template<typename T>
std::size_t hash_value(const tree<T>& tr)
{
    return boost::hash_range(tr.begin(), tr.end());
}

// Functor comparing the addresses of objects pointed by 
// tree iterators.  Useful for storing iterators in a std::map.
// (the tree has pointer to node, we use that to identify the
// tree node uniquely).
template<typename It>
struct obj_ptr_cmp {
    bool operator()(const It& lit, const It& rit) const {
        return ((void *) lit.node) < ((void *) rit.node);
    }
};

} //~namespace moses3

#endif // _OPENCOG_HASHING_H
