/*
 * moses3/util/oc_assert.h
 *
 * Copyright (C) 2002-2007 Novamente LLC
 * All Rights Reserved
 *
 * Written by Thiago Maia <thiago@vettatech.com>
 *            Andre Senna <senna@vettalabs.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef _OPENCOG_ASSERT_H
#define _OPENCOG_ASSERT_H

#include "exceptions.h"


// Macro OC_ASSERT corresponding to moses3::cassert(TRACE_INFO, ...)
// if IGNORE_OC_ASSERT is not defined, and ignored otherwise

// #define IGNORE_OC_ASSERT

#ifndef IGNORE_OC_ASSERT
#define OC_ASSERT(...) \
    moses3::cassert(TRACE_INFO, __VA_ARGS__)
#else
#define OC_ASSERT(...) \
    ((void)0)
#endif

namespace moses3 {

/**
 * cassert complet with message and trace info
 */
void cassert(const char * trace,  bool condition, const char * msg, ...);

/**
 * cassert with a string instead of const char* and vaargs
 */
void cassert(const char* trace, bool condition, const std::string& msg);

/**
 * cassert without message. Just trace information
 */
void cassert(const char * trace, bool condition);

} // namespace moses3

#endif
