/** oc_omp.h --- 
 *
 * Copyright (C) 2011 Nil Geisweiller
 *
 * Author: Nil Geisweiller <nilg@desktop>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#ifndef _OPENCOG_OC_OMP_H
#define _OPENCOG_OC_OMP_H

/**
 * File containing definitions to ease multi-threading support.
 *
 * The easiest way I found to do this is to use the parallel versions
 * of the STL algorithms.  To use them, include this file and put the
 * namespace OMP_ALGO in front of the algorithm.  For example,
 * OMP_ALGO::for_each(...) provides the parallel version of
 * std::for_each().  
 *
 * If the cpp define OC_OMP is defined, the parallel versions of the
 * std algorithms are used.   Disabling this define allows code to be
 * compiled with compilers that do not (yet) implement OMP, such as 
 * LLVM clang.
 */

// compile with multithread support
#define OC_OMP

#ifdef OC_OMP
#include <omp.h>
#include <parallel/algorithm>
#define OMP_ALGO __gnu_parallel
#else
#define OMP_ALGO std
#endif

namespace moses3 {

// setting the parallel env, such as number of threads, number of
// minimal iterations to parallelize
void setting_omp(unsigned num_threads, unsigned min_n = 50);

// returns the number of threads as configured by setting_omp
unsigned num_threads();

// split the number of jobs in 2. For instance if n_jobs is 3, then it
// returns <1, 2>. This function is convenient for parallalizing
// recursive functions
std::pair<unsigned, unsigned> split_jobs(unsigned n_jobs);

} // ~namespace moses3

#endif // _OPENCOG_OC_OMP_H
