/*
 * moses3/util/platform.h
 *
 * Copyright (C) 2002-2007 Novamente LLC
 * All Rights Reserved
 *
 * Written by Moshe Looks <moshe@metacog.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef _OPENCOG_PLATFORM_H
#define _OPENCOG_PLATFORM_H

#ifdef WIN32

#pragma warning(disable:4290)

#define strcasecmp _stricmp
#define snprintf _snprintf

#endif // WIN32

#include <stdio.h>
#include <string.h>
#include <string>
#include <stdint.h>

namespace moses3
{

struct eqstr {
    bool operator()(char *s1, char *s2) const {
        return strcmp(s1, s2) == 0;
    }
};

struct eqconststr {
    bool operator()(const char *s1, const char *s2) const {
        return strcmp(s1, s2) == 0;
    }
};

struct eqint {
    bool operator()(int s1, int s2) const {
        return s1 == s2;
    }
};

#ifdef __APPLE__
char*              __strtok_r(char *s1, const char *s2, char **lasts);
#endif

#ifdef WIN32_NOT_UNIX

#define M_PI 3.14159265358979323846

struct timezone {};

int                round(float x);
char*              __strtok_r(char *s1, const char *s2, char **lasts);
int                gettimeofday(struct timeval* tp, void* tzp);
void               usleep(unsigned useconds);
int                __getpid(void);
double             rint(double nr);
int                __dup2(int, int);
unsigned long long atoll(const char *str);
unsigned int       sleep(unsigned seconds);

#endif // ~WIN32_NOT_UNIX

size_t getMemUsage();

/// return the number of bytes of RAM
uint64_t getTotalRAM();

/// return the number of free bytes in RAM (excluding OS caches)
uint64_t getFreeRAM();

/// return the OS username
const char* getUserName();

} // namespace moses3

#endif // _OPENCOG_PLATFORM_H
