/** ranking.h --- 
 *
 * Copyright (C) 2012 Nil Geisweiller
 *
 * Author: Nil Geisweiller <nilg@desktop>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#ifndef _OPENCOG_RANKING_H
#define _OPENCOG_RANKING_H

#include "Counter.h"

namespace moses3 {

/**
 * Return a rank given a counter. Ties ranks are averaged.
 */
template<typename Key, typename FloatT>
Counter<Key, FloatT> ranking(const Counter<Key, FloatT>& counter) {
    Counter<Key, FloatT> res;
    FloatT lrank = 1;
    foreach(const auto& v, counter) {
        res.insert(res.end(), {v.first, (2*lrank + v.second - 1) / 2});
        lrank += v.second;
    }
    return res;
}

} // ~namespace moses3

#endif // _OPENCOG_RANKING_H
