/*
 * moses3/comboreduct/ant_combo_vocabulary/ant_combo_vocabulary.h
 *
 * Copyright (C) 2002-2008 Novamente LLC
 * All Rights Reserved
 *
 * Written by Nil Geisweiller
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#ifndef _ANT_COMBO_VOCABULARY_H
#define _ANT_COMBO_VOCABULARY_H

#include "ant_builtin_action.h"
#include "ant_perception.h"
#include "ant_action_symbol.h"
#include "ant_indefinite_object.h"
#include <moses3/comboreduct/combo/combo.h>

//this namespace is defined in order to break ambiguities
//and make coexist several vocabularies
namespace ant_combo {

using namespace moses3::combo;

//return a pointer to class base instance
//of a given action or perception enum
builtin_action get_instance(ant_builtin_action_enum);
perception get_instance(ant_perception_enum);
action_symbol get_instance(ant_action_symbol_enum);
indefinite_object get_instance(ant_indefinite_object_enum);

//get the enum corresponding to a builtin_action, perception or action_symbol
ant_builtin_action_enum get_enum(builtin_action);
ant_perception_enum get_enum(perception);
ant_action_symbol_enum get_enum(action_symbol);
ant_indefinite_object_enum get_enum(indefinite_object);
    
//this is added to be sure that the operators == and != between
//combo operator type and enum are used
//if not then it would easily lead to buggy situations because enum are cast to
//pointer
bool operator==(builtin_action, ant_builtin_action_enum);
bool operator==(ant_builtin_action_enum, builtin_action);
bool operator!=(builtin_action, ant_builtin_action_enum);
bool operator!=(ant_builtin_action_enum, builtin_action);
bool operator==(perception, ant_perception_enum);
bool operator==(ant_perception_enum, perception);
bool operator!=(perception, ant_perception_enum);
bool operator!=(ant_perception_enum, perception);
bool operator==(action_symbol, ant_action_symbol_enum);
bool operator==(ant_action_symbol_enum, action_symbol);
bool operator!=(action_symbol, ant_action_symbol_enum);
bool operator!=(ant_action_symbol_enum, action_symbol);
bool operator==(indefinite_object, ant_indefinite_object_enum);
bool operator==(ant_indefinite_object_enum, indefinite_object);
bool operator!=(indefinite_object, ant_indefinite_object_enum);
bool operator!=(ant_indefinite_object_enum, indefinite_object);

} // ~namespace ant_combo

#endif
