/*
 * moses3/comboreduct/combo/operator_base.h
 *
 * Copyright (C) 2002-2008 Novamente LLC
 * All Rights Reserved
 *
 * Written by Nil Geisweiller
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#ifndef _COMBO_OPERATOR_BASE_H
#define _COMBO_OPERATOR_BASE_H

#include "common_def.h"
#include "../type_checker/type_tree_def.h"

namespace moses3 { namespace combo {

  //that abstract class contains common methods
  //of builtin_action_base and perception_base
  class operator_base {
  public:
    virtual ~operator_base() {}

    //get_name
    virtual const std::string& get_name() const = 0;

    //type_tree
    virtual const type_tree& get_type_tree() const = 0;

    //helper methods for fast access type properties
    //number of arguments that takes the operator
    virtual arity_t arity() const = 0;
    //return the type node of the operator
    virtual type_tree get_output_type_tree() const = 0;
    //return the type tree of the input argument of index i
    virtual const type_tree& get_input_type_tree(arity_t i) const = 0;
  };

}} // ~namespace combo moses3

#endif
