/*
 * moses3/learning/moses/eda/scoring.h
 *
 * Copyright (C) 2002-2008 Novamente LLC
 * All Rights Reserved
 *
 * Written by Moshe Looks
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#ifndef _EDA_SCORING_H
#define _EDA_SCORING_H

#include "eda.h"
#include <moses3/util/functional.h>

namespace moses3 { 
namespace moses {

template<typename ScoreT>
struct scored_instance : public tagged_item<instance, ScoreT>
{
    typedef tagged_item<instance, ScoreT> super;

    scored_instance(const instance& i, const ScoreT& s) : super(i, s) { }
    scored_instance(const instance& i) : super(i) { }
    scored_instance() { }
    template<class T1, class T2>
    scored_instance(const std::pair<T1, T2>& p) : super(p) { }

    bool operator<(const scored_instance& other) const
    {
        return this->second < other.second;
    }
    bool operator>(const scored_instance& other) const
    {
        return this->second > other.second;
    }
};

/**
 * score_instance takes an instance and a scoring functor and returns
 * the scored_instance
 *
 * @param inst candidate instance
 * @param score scoring functor
 * @return scored_instance
 */
template<typename Scoring>
scored_instance < typename result_of < Scoring(instance) >::type >
score_instance(const instance& inst, const Scoring& score)
{
    return scored_instance < typename result_of < Scoring(instance) >::type >
        (inst, score(inst));
}

} // ~namespace moses
} // ~namespace moses3

#endif
