/** moses_exec_def.h --- 
 *
 * Copyright (C) 2010 Novamente LLC
 *
 * Author: Nil Geisweiller
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#ifndef _OPENCOG_MOSES_EXEC_DEF_H
#define _OPENCOG_MOSES_EXEC_DEF_H

#include <boost/program_options.hpp>
#include <boost/lexical_cast.hpp>

#include <moses3/util/iostreamContainer.h>

namespace moses3 { namespace moses {

using namespace std;

// program option names and abbreviations (one letter). If the
// abbreviation is empty there is none. For their meanings see
// options_description in moses-exec.cc.
//
// Available abbreviations are: O
static const pair<string, string> rand_seed_opt("random-seed", "r");
static const pair<string, string> input_data_file_opt("input-file", "i");
static const pair<string, string> target_feature_opt("target-feature", "u");
static const pair<string, string> problem_opt("problem", "H");
static const pair<string, string> combo_str_opt("combo-program", "y");
static const pair<string, string> problem_size_opt("problem-size", "k");
static const pair<string, string> nsamples_opt("nsamples", "b");
static const pair<string, string> min_rand_input_opt("min-rand-input", "q");
static const pair<string, string> max_rand_input_opt("max-rand-input", "w");
static const pair<string, string> max_evals_opt("max-evals", "m");
static const pair<string, string> result_count_opt("result-count", "c");
static const pair<string, string> output_score_opt("output-score", "S");
static const pair<string, string> output_penalty_opt("output-penalty", "x");
static const pair<string, string> output_bscore_opt("output-bscore", "t");
static const pair<string, string> output_only_best_opt("output-only-best", "C");
static const pair<string, string> output_eval_number_opt("output-eval-number", "V");
static const pair<string, string> output_with_labels_opt("output-with-labels", "W");
static const pair<string, string> output_file_opt("output-file", "o");
static const pair<string, string> max_gens_opt("max-gens", "g");
static const pair<string, string> log_level_opt("log-level", "l");
static const pair<string, string> log_file_opt("log-file", "f");
static const string default_log_file_prefix = "moses";
static const string default_log_file_suffix = "log";
static const string default_log_file = default_log_file_prefix + "." + default_log_file_suffix;
static const pair<string, string> log_file_dep_opt_opt("log-file-dep-opt", "F");
static const pair<string, string> noise_opt("noise", "p");
static const pair<string, string> include_only_ops_str_opt("include-only-operator", "N");
static const pair<string, string> ignore_ops_str_opt("ignore-operator", "n");
static const pair<string, string> ignore_feature_str_opt("ignore-feature", "Y");
static const pair<string, string> opt_algo_opt("opt-algo", "a");
static const pair<string, string> exemplars_str_opt("exemplar", "e");
static const pair<string, string> reduct_candidate_effort_opt("reduct-candidate-effort", "E");
static const pair<string, string> reduct_knob_building_effort_opt("reduct-knob-building-effort", "B");
static const pair<string, string> reduce_all_opt("reduce-all", "d");
static const pair<string, string> cache_size_opt("cache-size", "s");
static const pair<string, string> jobs_opt("jobs", "j");
static const string job_seperator(":");
static const string localhost("localhost");
static const pair<string, string> weighted_accuracy_opt("weighted-accuracy", "G");
static const pair<string, string> pop_size_ratio_opt("pop-size-ratio", "P");
static const pair<string, string> max_score_opt("max-score", "A");
static const pair<string, string> max_dist_opt("max-dist", "D");
static const pair<string, string> max_candidates_opt("max-candidates", "M");
static const pair<string, string> include_dominated_opt("include-dominated", "I");
static const pair<string, string> complexity_temperature_opt("complexity-temperature", "v");
static const pair<string, string> complexity_ratio_opt("complexity-ratio", "z");
static const pair<string, string> discretize_threshold_opt("discretize-threshold", "R");

// options specific to a particular fitness function
static const pair<string, string> ip_kld_weight_opt("ip_kld_weight", "K");
static const pair<string, string> ip_skewness_weight_opt("ip_skewness_weight", "J");
static const pair<string, string> ip_stdU_weight_opt("ip_stdU_weight", "U");
static const pair<string, string> ip_skew_U_weight_opt("ip_skew_U_weight", "X");

static const pair<string, string> alpha_opt("alpha", "Q");

// options specific to a particular optimization algorithm
static const pair<string, string> hc_single_step_opt("hc-single-step", "L");
static const pair<string, string> hc_widen_search_opt("hc-widen-search", "T");
static const pair<string, string> hc_crossover_opt("hc-crossover", "Z");

// Returns a string interpretable by Boost.Program_options
// "name,abbreviation"
string opt_desc_str(const pair<string, string>& opt);

} // ~namespace moses
} // ~namespace moses3

#endif // _OPENCOG_MOSES_OPTIONS_NAMES_H
