/*
 * moses3/learning/moses/moses/moses_params.h
 *
 * Copyright (C) 2002-2008 Novamente LLC
 * All Rights Reserved
 *
 * Written by Moshe Looks
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#ifndef _MOSES_MOSES_PARAMS_H
#define _MOSES_MOSES_PARAMS_H

#include <atomic>
#include <map>
#include <boost/program_options/variables_map.hpp>

namespace moses3 {
namespace moses {

/// A map between hostname and number of jobs allocated.
typedef std::map<string, unsigned> jobs_t;

/**
 * parameters to decide how to run moses
 */
struct moses_parameters
{
    moses_parameters(const boost::program_options::variables_map& _vm =
                           boost::program_options::variables_map(),
                     const jobs_t& _jobs = jobs_t(),
                     bool _local = true,
                     int _max_evals = 10000,
                     int _max_gens = -1,
                     score_t _max_score = 0,
                     int _max_cnd_output = -1)
        : local(_local), mpi(false), force_feed(false), jobs(_jobs), vm(_vm),
          max_evals(_max_evals), max_gens(_max_gens), max_score(_max_score),
          max_cnd_output(_max_cnd_output)
    {}

    // Distributed solver control.
    bool local;
    bool mpi;
    bool force_feed;
    const jobs_t& jobs;
    const boost::program_options::variables_map& vm;

    // total maximun number of evals
    int max_evals;
    
    // the max number of demes to create and optimize, if negative,
    // then no limit
    int max_gens;
    
    // the max score
    score_t max_score;

    // limit amount of time that process should run
    time_t max_time;

    // the maximum number of candidates to output (if negative then it
    // output all candidates
    int max_cnd_output;
};

/**
 * Keep track of miscellaneous solver statistics
 */
struct moses_statistics
{
    moses_statistics() : n_evals(0), n_expansions(0), elapsed_secs(0)
    {}

    // total number of scoring function evaluations
    std::atomic<int> n_evals;

    // total number of deme expansions (generations)
    std::atomic<int> n_expansions;

    // elapsed wall-clock time, in seconds, since start of moses.
    double elapsed_secs;
};


} // ~namespace moses
} // ~namespace moses3

#endif
