/*
 * moses3/learning/moses/optimization/univariate.h
 *
 * Copyright (C) 2002-2008 Novamente LLC
 * Copyright (C) 2012 Poulin Holdings
 * All Rights Reserved
 *
 * Written by Moshe Looks
 *            Predrag Janicic
 *            Nil Geisweiller
 *            Xiaohui Liu
 *            Linas Vepstas
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#ifndef _MOSES_UNIVARIATE_H
#define _MOSES_UNIVARIATE_H

#include "../representation/instance_set.h"
#include "optimization.h"

namespace moses3 { namespace moses {

// Parameters specific to EDA optimization
struct eda_parameters
{
    eda_parameters() :
        selection(2),          //if <=1, truncation selection ratio,
                               //if >1, tournament selection size (should be int)
        selection_ratio(1),    //ratio of population size selected for modeling

        replacement_ratio(0.5),//ratio of population size sampled and integrated

        model_complexity(1)    //model parsimony term log(N)*model_complexity
    {}

    bool is_tournament_selection() {
        return selection > 1;
    }
    bool is_truncation_selection() {
        return selection <= 1;
    }

    double selection;
    double selection_ratio;
    double replacement_ratio;
    double model_complexity;
};

struct univariate_optimization : optimizer_base
{
    univariate_optimization(const optim_parameters& op = optim_parameters(),
                            const eda_parameters& ep = eda_parameters())
        : opt_params(op), eda_params(ep) {}

    // Return # of evaluations actually performed
    unsigned operator()(deme_t& deme,
                        const iscorer_base& iscorer,
                        unsigned max_evals,
                        time_t max_time);

    optim_parameters opt_params;
    eda_parameters eda_params;
};

} // ~namespace moses
} // ~namespace moses3

#endif
