/*
 * moses3/comboreduct/reduct/branch_rules.h
 *
 * Copyright (C) 2002-2008 Novamente LLC
 * Copyright (C) 2012 Poulin Holdings
 * All Rights Reserved
 *
 * Written by Nil Geisweiller, Linas Vepstas
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#ifndef _REDUCT_BRANCH_RULES_H
#define _REDUCT_BRANCH_RULES_H

#include "reduct.h"

namespace moses3 { namespace reduct {

/// Apply boolean reduction to each predicate
/// (same class structure as reduce_impulse_arg)
struct reduce_cond_arg : public crule<reduce_cond_arg>
{
    int reduct_effort;
    const vertex_set &ignore_ops;
    reduce_cond_arg(int effort, const vertex_set &igop)
        : crule<reduce_cond_arg>::crule("reduce_cond_arg"),
          reduct_effort(effort), ignore_ops(igop) {}
    void operator()(combo_tree& tr, combo_tree::iterator it) const;
}; 


/// If the last two consequents are identical, omit the last clause.
/// cond(p1 x1 ... pn xn p x x) -> cond(p1 x1 ... pn xn x)
/// cond(v) -> v
struct reduce_cond_else : public crule<reduce_cond_else> 
{
    reduce_cond_else() 
        : crule<reduce_cond_else>::crule("reduce_cond_else") {}
    void operator()(combo_tree& tr, combo_tree::iterator it) const;
};


/// If two neighboring consequents are identical, merge them.
/// cond(... p x q x ...) -> cond (... or(p q) x ...)
struct reduce_cond_adjacent : public crule<reduce_cond_adjacent> 
{
    reduce_cond_adjacent() 
        : crule<reduce_cond_adjacent>::crule("reduce_cond_adjacent") {}
    void operator()(combo_tree& tr, combo_tree::iterator it) const;
};

/// Reduce cond's with constant predicates.
///
/// Any predicate that is true truncates the rest of the condition.
/// cond(true x1 ... pn xn y) -> x1
/// cond(p1 x1 ... true xk ... pn xn y) -> cond(p1 x1 ... p{k-1} x{k-1}
/// xk)
///
/// Any predicate that is false causes the whole clause to fall out.
/// cond(false x1 ... pn xn y) -> cond(p2 x2 ... pn xn y)
/// cond(p1 x1 ... false xk ... pn xn y) -> 
///                 cond(p1 x1 ... p{k-1} x{k-1} p{k+1} x{k+1} ... pn xn
///                 y)
///
/// This can happen along the way.
/// cond(v) -> v
///
/// This partially obsoletes reduce_contin_if
struct reduce_cond_const : public crule<reduce_cond_const> 
{
    reduce_cond_const() 
        : crule<reduce_cond_const>::crule("reduce_cond_const") {}
    void operator()(combo_tree& tr, combo_tree::iterator it) const;
};


} // ~namespace reduct
} // ~namespace moses3

#endif
