/*
 * moses3/comboreduct/reduct/general_rules.h
 *
 * Copyright (C) 2002-2008 Novamente LLC
 * All Rights Reserved
 *
 * Written by Moshe Looks
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#ifndef _REDUCT_GENERAL_RULES_H
#define _REDUCT_GENERAL_RULES_H

#include <moses3/util/RandGen.h>

#include "reduct.h"
#include "../interpreter/eval.h"

namespace moses3 { namespace reduct {

// Flattens all associative functions: f(a,f(b,c)) -> f(a,b,c)
// Note that level is recursive that is f(a,f(b,f(c,d))) -> f(a,b,c,d)
struct level : public crule<level>
{ 
    level() : crule<level>::crule("level") {}
    void operator()(combo_tree& tr,combo_tree::iterator it) const;
};

// Evaluates sub-expressions when possible.
// If an operator is commutative, op(const,var,const) will become
// op(op(const,const),var), e.g., +(2,x,1) => +(3,x)
struct eval_constants : public crule<eval_constants>
{ 
    Evaluator* evaluator;
    eval_constants(Evaluator* e = NULL)
        : crule<eval_constants>::crule("eval_constants"), evaluator(e) {}
    void operator()(combo_tree& tr,combo_tree::iterator it) const;
};

// Reorder children of commutative operators (should be applied upwards).
struct reorder_commutative : public crule<reorder_commutative>
{
    reorder_commutative()
        : crule<reorder_commutative>::crule("reorder_commutative") {}
    void operator()(combo_tree& tr,combo_tree::iterator it) const;
};

// Get rid of subtrees marked with a null_vertex in their roots.
struct remove_null_vertices : public crule<remove_null_vertices>
{
    remove_null_vertices()
        : crule<remove_null_vertices>::crule("remove_null_vertices") {}
    void operator()(combo_tree& tr,combo_tree::iterator it) const;
};

// Remove all assumptions.
struct remove_all_assumptions : public crule<remove_all_assumptions>
{
    remove_all_assumptions()
        : crule<remove_all_assumptions>::crule("remove_all_assumptions") {}
    void operator()(combo_tree& tr,combo_tree::iterator it) const;
};

} // ~namespace reduct
} // ~namespace moses3

#endif
