/*
 * moses3/comboreduct/reduct/perception_rules.h
 *
 * Copyright (C) 2002-2008 Novamente LLC
 * All Rights Reserved
 *
 * Written by Nil Geisweiller
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#ifndef _REDUCT_PERCEPTION_RULES_H
#define _REDUCT_PERCEPTION_RULES_H

#include "reduct.h"
#include <moses3/comboreduct/type_checker/type_tree.h>

namespace moses3 { namespace reduct {
  
//add in the set of assumptions such knowledge
//f(x, z) <= max{d(x, y), d(y, z)}
//when encounting f(x, y) and f(y, z) and d is ultrametric
struct reduce_ultrametric : public crule<reduce_ultrametric> {
    reduce_ultrametric() : crule<reduce_ultrametric>::crule("reduce_ultrametric") {}
    void operator()(combo_tree& tr,combo_tree::iterator it) const;
};
 
//add in the set of assumptions such knowledge
//f(x, z)
//when encounting f(x, y) and f(y, z)
struct reduce_transitive : public crule<reduce_transitive> {
    reduce_transitive() : crule<reduce_transitive>::crule("reduce_transitive") {}
    void operator()(combo_tree& tr,combo_tree::iterator it) const;
};

//reduce f(x,x) -> true iff f is reflexive
struct reduce_reflexive : public crule<reduce_reflexive> {
    reduce_reflexive() : crule<reduce_reflexive>::crule("reduce_reflexive") {}
    void operator()(combo_tree& tr,combo_tree::iterator it) const;
};

//reduce f(x,x) -> false iff f is irreflexive
struct reduce_irreflexive : public crule<reduce_irreflexive> {
    reduce_irreflexive() : crule<reduce_irreflexive>::crule("reduce_irreflexive") {}
    void operator()(combo_tree& tr,combo_tree::iterator it) const;
};


//add f(y,x) in the set of assumptions if f(x,y)
//Note that although it seems that since a symmetric
//operator is commutative there is no need of this rule
//it is not true because this rule permits to combines
//different properties of a relation, as for instance
//if the relation is both symmetric and transitive
//then the combination of reduce_transitive and
//reduce_symmetric can permit more reductions
//than with associative reduction.
struct reduce_symmetric : public crule<reduce_symmetric> {
    reduce_symmetric() : crule<reduce_symmetric>::crule("reduce_symmetric") {}
    void operator()(combo_tree& tr,combo_tree::iterator it) const;
};


//reduce f(x,y) -> 0 iff f verifies the property of identity of indiscernibles
struct reduce_identity_of_indiscernibles : public crule<reduce_identity_of_indiscernibles> {
    reduce_identity_of_indiscernibles() : crule<reduce_identity_of_indiscernibles>::crule("reduce_identity_of_indiscernibles") {}
    void operator()(combo_tree& tr,combo_tree::iterator it) const;
};

} // ~namespace reduct
} // ~namespace moses3

#endif
