/*
 * moses3/util/lazy_random_selector.h
 *
 * Copyright (C) 2002-2007 Novamente LLC
 * All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef _OPENCOG_LAZY_RANDOM_SELECTOR_H
#define _OPENCOG_LAZY_RANDOM_SELECTOR_H

#include "lazy_selector.h"
#include "RandGen.h"
#include "mt19937ar.h"

namespace moses3
{

// a lazy random selector without replacement -
// lets you select m random integers in [0,n) without replacement
// (i.e. it never selects twice the same number)
// each in O(1) and only uses O(m) memory - useful where n is much larger than m
struct lazy_random_selector : public lazy_selector {
    lazy_random_selector(unsigned int n, moses3::RandGen& _rng = randGen())
        : lazy_selector(n), rng(_rng) { }
    unsigned int select();
private:
    moses3::RandGen& rng;
};

} //~namespace moses3

#endif
