/** log_prog_name.cc --- 
 *
 * Copyright (C) 2011 OpenCog Foundation
 *
 * Author: Nil Geisweiller
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "log_prog_name.h"
#include "oc_assert.h"

namespace moses3 {

using namespace std;

/**
 * This file contains a little function to determine a log file name
 * automatically depending on its boost.program_option.
 */

string to_string(const boost::program_options::variable_value& vv,
                 string separator)
{
    string res;
    if(!(to_string<int>(vv, res, separator)
         || to_string<unsigned int>(vv, res, separator)
         || to_string<long>(vv, res, separator)
         || to_string<unsigned long>(vv, res, separator)
         || to_string<float>(vv, res, separator)
         || to_string<double>(vv, res, separator)
         || to_string<bool>(vv, res, separator)
         || to_string<string>(vv, res, separator)))
        cerr << "type not handled yet" << endl;
    return res;
}

/**
 * determine a log file name automatically depending on its
 * boost.program_option.
 */
string determine_log_name(const string& log_file_prefix,
                          const boost::program_options::variables_map& vm,
                          const set<string>& ignore_opt,
                          const string& log_file_suffix)
{
    const static unsigned int max_filename_size = 255;

    string log_file = log_file_prefix;
    for(boost::program_options::variables_map::const_iterator it = vm.begin();
        it != vm.end(); it++)
        // we ignore the options in ignore_opt and any default one
        if(ignore_opt.find(it->first) == ignore_opt.end()
           && !it->second.defaulted()) {
            string str = string("_") + it->first + "_" + to_string(it->second);
            // this is because OSs usually do not handle file name
            // above 255 chars
            unsigned int expected_max_size =
                log_file.size()+str.size()+log_file_suffix.size();
            if(expected_max_size < max_filename_size) {
                log_file += str;
            }
        }
    log_file += log_file_suffix;
    // replace / by d because unix file name cannot have / in it
    replace(log_file.begin(), log_file.end(), '/', 'd');
    OC_ASSERT(log_file.size() <= max_filename_size);
    return log_file;
}

} // ~namespace moses3
