/** log_prog_name.h --- 
 *
 * Copyright (C) 2011 OpenCog Foundation
 *
 * Author: Nil Geisweiller
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef _OPENCOG_LOG_PROG_NAME_H
#define _OPENCOG_LOG_PROG_NAME_H

#include <boost/program_options.hpp>
#include "iostreamContainer.h"

namespace moses3 {

/**
 * This file contains a little function to determine a log file name
 * automatically depending on its boost.program_option.
 */


// used to convert program option argument to string.
// @todo: ugly, it is likely something better can be done using
// boost::program_options API
template<typename T>
bool to_string(const boost::program_options::variable_value& vv,
               std::string& str,
               std::string separator)
{
    if(vv.value().type() == typeid(T)) {
        str = boost::lexical_cast<std::string>(vv.as<T>());
        return true;
    } else if(vv.value().type() == typeid(std::vector<T>)) {
        str = moses3::containerToStr(vv.as<std::vector<T> >(), separator.c_str());
        return true;
    }
    return false;
}
std::string to_string(const boost::program_options::variable_value& vv,
                      std::string separator = "_");

/**
 * determine a log file name automatically depending on its
 * boost.program_option.
 */
std::string determine_log_name(const std::string& log_file_prefix,
                               const boost::program_options::variables_map& vm,
                               const std::set<std::string>& ignore_opt,
                               const std::string& log_file_suffix = ".log");

} // ~namespace moses3

#endif // _OPENCOG_LOG_PROG_NAME_H
