/** oc_omp.cc --- 
 *
 * Copyright (C) 2011 Nil Geisweiller
 *
 * Author: Nil Geisweiller <nilg@desktop>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "oc_omp.h"

namespace moses3 {

void setting_omp(unsigned num_threads, unsigned min_n) {
#ifdef OC_OMP
    omp_set_dynamic(false);
    omp_set_num_threads(num_threads);
    __gnu_parallel::_Settings gps;
    gps.transform_minimal_n = min_n;
    gps.for_each_minimal_n = min_n;
    gps.replace_minimal_n = min_n;
    __gnu_parallel::_Settings::set(gps);
#endif
}

unsigned num_threads() {
#ifdef OC_OMP
    return omp_get_max_threads();
#else
    return 1;
#endif
}

std::pair<unsigned, unsigned> split_jobs(unsigned n_jobs) {
    unsigned n_jobs1 = n_jobs / 2;
    unsigned n_jobs2 = std::max(1U, n_jobs - n_jobs1);
    return {n_jobs1, n_jobs2};
}

}
